/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (c) 2017 Google, Inc
 */
#ifndef _ASM_ARCH_PERIPH_H
#define _ASM_ARCH_PERIPH_H

/*
 * Peripherals supported by the hardware.
 * These are used to specify pinctrl settings.
 */

enum periph_id {
	PERIPH_ID_UART1,
	PERIPH_ID_UART2,
	PERIPH_ID_UART3,
	PERIPH_ID_UART4,
	PERIPH_ID_LPC,
	PERIPH_ID_PWM0,
	PERIPH_ID_PWM1,
	PERIPH_ID_PWM2,
	PERIPH_ID_PWM3,
	PERIPH_ID_PWM4,
	PERIPH_ID_PWM5,
	PERIPH_ID_PWM6,
	PERIPH_ID_PWM7,
	PERIPH_ID_PWM8,
	PERIPH_ID_MAC1,
	PERIPH_ID_MAC2,
	PERIPH_ID_VIDEO,
	PERIPH_ID_SPI1,
	PERIPH_ID_SPI2,
	PERIPH_ID_I2C1,
	PERIPH_ID_I2C2,
	PERIPH_ID_I2C3,
	PERIPH_ID_I2C4,
	PERIPH_ID_I2C5,
	PERIPH_ID_I2C6,
	PERIPH_ID_I2C7,
	PERIPH_ID_I2C8,
	PERIPH_ID_I2C9,
	PERIPH_ID_I2C10,
	PERIPH_ID_I2C11,
	PERIPH_ID_I2C12,
	PERIPH_ID_I2C13,
	PERIPH_ID_I2C14,
	PERIPH_ID_SD1,
	PERIPH_ID_SD2,
};

#endif  /* _ASM_ARCH_SCU_AST2500_H */
