/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2004-2014
 * Texas Instruments, <www.ti.com>
 *
 * Some changes copyright (C) 2007 Sergey Kubushyn <ksi@koi8.net>
 */
#ifndef _I2C_DEFS_H_
#define _I2C_DEFS_H_

#define I2C_BASE		0x01c22000

#endif
