/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Fake include for LzmaDec.h
 *
 * Copyright (C) 2007-2009 Industrie Dial Face S.p.A.
 * Luigi 'Comio' Mantellini (luigi.mantellini@idf-hit.com)
 *
 */

#ifndef __LZMADEC_H__FAKE__
#define __LZMADEC_H__FAKE__

#include "../../lib/lzma/LzmaDec.h"

#endif
