/*
 * Port I Masks
 */

#ifndef __BFIN_PERIPHERAL_PORT_I__
#define __BFIN_PERIPHERAL_PORT_I__

#define PI0		(1 << 0)
#define PI1		(1 << 1)
#define PI2		(1 << 2)
#define PI3		(1 << 3)
#define PI4		(1 << 4)
#define PI5		(1 << 5)
#define PI6		(1 << 6)
#define PI7		(1 << 7)
#define PI8		(1 << 8)
#define PI9		(1 << 9)
#define PI10		(1 << 10)
#define PI11		(1 << 11)
#define PI12		(1 << 12)
#define PI13		(1 << 13)
#define PI14		(1 << 14)
#define PI15		(1 << 15)

#endif
