/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2017 Álvaro Fernández Rojas <noltari@gmail.com>
 *
 * Derived from linux/arch/mips/include/asm/mach-bcm63xx/bcm63xx_regs.h
 */

#ifndef __DT_BINDINGS_CLOCK_BCM6338_H
#define __DT_BINDINGS_CLOCK_BCM6338_H

#define BCM6338_CLK_ADSL	0
#define BCM6338_CLK_MPI		1
#define BCM6338_CLK_SDRAM	2
#define BCM6338_CLK_ENET	4
#define BCM6338_CLK_SAR		5
#define BCM6338_CLK_SPI		9

#endif /* __DT_BINDINGS_CLOCK_BCM6338_H */
