/* DO NOT EDIT THIS FILE
 * Automatically generated by generate-def-headers.xsl
 * DO NOT EDIT THIS FILE
 */

#ifndef __BFIN_DEF_ADSP_EDN_core__
#define __BFIN_DEF_ADSP_EDN_core__

#define WPIACTL                        0xFFE07000
#define WPIA0                          0xFFE07040
#define WPIA1                          0xFFE07044
#define WPIA2                          0xFFE07048
#define WPIA3                          0xFFE0704C
#define WPIA4                          0xFFE07050
#define WPIA5                          0xFFE07054
#define WPIACNT0                       0xFFE07080
#define WPIACNT1                       0xFFE07084
#define WPIACNT2                       0xFFE07088
#define WPIACNT3                       0xFFE0708C
#define WPIACNT4                       0xFFE07090
#define WPIACNT5                       0xFFE07094
#define WPDACTL                        0xFFE07100
#define WPDA0                          0xFFE07140
#define WPDA1                          0xFFE07144
#define WPDACNT0                       0xFFE07180
#define WPDACNT1                       0xFFE07184
#define WPSTAT                         0xFFE07200
#define DSPID                          0xFFE05000
#define DBGSTAT                        0xFFE05008

#endif /* __BFIN_DEF_ADSP_EDN_core__ */
