/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2017 Álvaro Fernández Rojas <noltari@gmail.com>
 *
 * Derived from linux/arch/mips/include/asm/mach-bcm63xx/bcm63xx_regs.h
 */

#ifndef __DT_BINDINGS_RESET_BCM6338_H
#define __DT_BINDINGS_RESET_BCM6338_H

#define BCM6338_RST_SPI		0
#define BCM6338_RST_ENET	2
#define BCM6338_RST_USBH	3
#define BCM6338_RST_USBS	4
#define BCM6338_RST_ADSL	5
#define BCM6338_RST_DMAMEM	6
#define BCM6338_RST_SAR		7
#define BCM6338_RST_ACLC	8
#define BCM6338_RST_ADSL_MIPS	10

#endif /* __DT_BINDINGS_RESET_BCM6338_H */
