/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2018 Álvaro Fernández Rojas <noltari@gmail.com>
 */

#ifndef __DT_BINDINGS_POWER_DOMAIN_BCM6318_H
#define __DT_BINDINGS_POWER_DOMAIN_BCM6318_H

#define BCM6318_PWR_PCIE	0
#define BCM6318_PWR_USB		1

#endif /* __DT_BINDINGS_POWER_DOMAIN_BCM6318_H */
