/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2019 Rockchip Electronics Co., Ltd
 */

#ifndef __ASM_ARCH_GPIO_H__
#define __ASM_ARCH_GPIO_H__

#include <asm/arch-rockchip/gpio.h>

#endif
