/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2004, Psyent Corporation <www.psyent.com>
 * Scott McNutt <smcnutt@psyent.com>
 */

#ifndef __ASM_NIOS2_BITOPS_H_
#define __ASM_NIOS2_BITOPS_H_

/* copied from linux-2.6/include/asm-generic/bitops */
#include <asm/bitops/atomic.h>
#include <asm/bitops/non-atomic.h>
#include <asm/bitops/ffs.h>

#include <asm-generic/bitops/fls.h>
#include <asm-generic/bitops/__fls.h>
#include <asm-generic/bitops/fls64.h>
#include <asm-generic/bitops/__ffs.h>

#endif /* __ASM_NIOS2_BITOPS_H */
