/*
 * Copyright 2016 Maxime Ripard
 *
 * Maxime Ripard <maxime.ripard@free-electrons.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _RST_SUN5I_H_
#define _RST_SUN5I_H_

#define RST_USB_PHY0	0
#define RST_USB_PHY1	1
#define RST_GPS		2
#define RST_DE_BE	3
#define RST_DE_FE	4
#define RST_TVE		5
#define RST_LCD		6
#define RST_CSI		7
#define RST_VE		8
#define RST_GPU		9
#define RST_IEP		10

#endif /* _RST_SUN5I_H_ */
