/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2014 Broadcom Corporation.
 */

#ifndef __ARMPLL_H
#define __ARMPLL_H

#include <linux/types.h>

uint32_t armpll_config(uint32_t clkmhz);

#endif /*__ARMPLL_H */
