/*
 * Port C Masks
 */

#ifndef __BFIN_PERIPHERAL_PORT_C__
#define __BFIN_PERIPHERAL_PORT_C__

#define PC0		(1 << 0)
#define PC1		(1 << 1)
#define PC2		(1 << 2)
#define PC3		(1 << 3)
#define PC4		(1 << 4)
#define PC5		(1 << 5)
#define PC6		(1 << 6)
#define PC7		(1 << 7)
#define PC8		(1 << 8)
#define PC9		(1 << 9)
#define PC10		(1 << 10)
#define PC11		(1 << 11)
#define PC12		(1 << 12)
#define PC13		(1 << 13)
#define PC14		(1 << 14)
#define PC15		(1 << 15)

#endif
