/*
 * Port F Masks
 */

#ifndef __BFIN_PERIPHERAL_PORT_F__
#define __BFIN_PERIPHERAL_PORT_F__

#define PF0		(1 << 0)
#define PF1		(1 << 1)
#define PF2		(1 << 2)
#define PF3		(1 << 3)
#define PF4		(1 << 4)
#define PF5		(1 << 5)
#define PF6		(1 << 6)
#define PF7		(1 << 7)
#define PF8		(1 << 8)
#define PF9		(1 << 9)
#define PF10		(1 << 10)
#define PF11		(1 << 11)
#define PF12		(1 << 12)
#define PF13		(1 << 13)
#define PF14		(1 << 14)
#define PF15		(1 << 15)

#endif
