/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2016 BayLibre, SAS
 * Author: Maxime Jourdan <mjourdan@baylibre.com>
 */

#ifndef __MESON_VPU_H__
#define __MESON_VPU_H__

/* Allow reserving the framebuffer memory region */
void meson_vpu_rsv_fb(void *fdt);

#endif /* __MESON_VPU_H__ */
