/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Empty gpio.h
 *
 * This file must stay as arch/arm/include/asm/gpio.h requires it.
 *
 * Copyright (c) 2019 Sartura Ltd.
 *
 * Author: Robert Marko <robert.marko@sartura.hr>
 */
