/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2020 MediaTek Inc.
 *
 * Author:  Weijie Gao <weijie.gao@mediatek.com>
 */

#ifndef _MTMIPS_SERIAL_H_
#define _MTMIPS_SERIAL_H_

void mtmips_spl_serial_init(void);

#endif /* _MTMIPS_SERIAL_H_ */
