/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Tegra pulse width frequency modulator definitions
 *
 * Copyright (c) 2011 The Chromium OS Authors.
 */

#ifndef __ASM_ARCH_TEGRA20_PWM_H
#define __ASM_ARCH_TEGRA20_PWM_H

#include <asm/arch-tegra/pwm.h>

#endif	/* __ASM_ARCH_TEGRA20_PWM_H */
