/* SPDX-License-Identifier: MIT */
/*
 * Define target-specific memory model
 * Copyright (c) 2008 Fabrice Bellard
 * Copyright (c) 2008 Andrzej Zaborowski
 */

#ifndef TCG_TARGET_MO_H
#define TCG_TARGET_MO_H

#define TCG_TARGET_DEFAULT_MO  0

#endif
