/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2018  Cisco Systems, Inc.
 *
 * Author: Thomas Fitzsimmons <fitzsim@fitzsim.org>
 */

#ifndef _BCMSTB_GPIO_H
#define _BCMSTB_GPIO_H

#endif /* _BCMSTB_GPIO_H */
