/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * K3: Common SoC clock definitions.
 *
 * (C) Copyright (C) 2018 Texas Instruments Incorporated - http://www.ti.com/
 */
#ifndef __ASM_ARCH_CLOCK_H
#define __ASM_ARCH_CLOCK_H

#include <config.h>

/* Clock Defines */
#define V_OSCK				24000000
#define V_SCLK				V_OSCK

#endif /* __ASM_ARCH_CLOCK_H */
