/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (c) 2017 Rockchip Electronics Co., Ltd
 */

#ifndef __CONFIGS_PX5_EVB_H
#define __CONFIGS_PX5_EVB_H

#include <configs/rk3368_common.h>

#endif
